# Copyright (c) 2013 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Defines database models for Barbican
"""

from sqlalchemy import Column, Integer, String, BigInteger
from sqlalchemy.ext.compiler import compiles
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import ForeignKey, DateTime, Boolean, Text, LargeBinary
from sqlalchemy.orm import relationship, backref, object_mapper
from sqlalchemy import Index, UniqueConstraint

from barbican.openstack.common import timeutils
from barbican.openstack.common import uuidutils
from barbican.openstack.common import jsonutils as json
from barbican.common import utils

LOG = utils.getLogger(__name__)
BASE = declarative_base()


# Allowed entity states
class States(object):
    PENDING = 'PENDING'
    ACTIVE = 'ACTIVE'

    @classmethod
    def is_valid(self, state_to_test):
        """Tests if a state is a valid one."""
        return state_to_test in self.__dict__


@compiles(BigInteger, 'sqlite')
def compile_big_int_sqlite(type_, compiler, **kw):
    return 'INTEGER'


class ModelBase(object):
    """Base class for Nova and Barbican Models"""
    __table_args__ = {'mysql_engine': 'InnoDB'}
    __table_initialized__ = False
    __protected_attributes__ = set([
        "created_at", "updated_at", "deleted_at", "deleted"])

    id = Column(String(36), primary_key=True, default=uuidutils.generate_uuid)

    created_at = Column(DateTime, default=timeutils.utcnow,
                        nullable=False)
    updated_at = Column(DateTime, default=timeutils.utcnow,
                        nullable=False, onupdate=timeutils.utcnow)
    deleted_at = Column(DateTime)
    deleted = Column(Boolean, nullable=False, default=False)

    status = Column(String(20), nullable=False, default=States.PENDING)

    def save(self, session=None):
        """Save this object"""
        # import api here to prevent circular dependency problem
        import barbican.model.repositories
        session = session or barbican.model.repositories.get_session()
        session.add(self)
        session.flush()

    def delete(self, session=None):
        """Delete this object"""
        import barbican.model.repositories
        session = session or barbican.model.repositories.get_session()
        self.deleted = True
        self.deleted_at = timeutils.utcnow()
        self.save(session=session)

        self._do_delete_children(session)

    def _do_delete_children(self, session):
        """
        Sub-class hook: delete children relationships.
        """
        pass

    def update(self, values):
        """dict.update() behaviour."""
        for k, v in values.iteritems():
            self[k] = v

    def __setitem__(self, key, value):
        setattr(self, key, value)

    def __getitem__(self, key):
        return getattr(self, key)

    def __iter__(self):
        self._i = iter(object_mapper(self).columns)
        return self

    def next(self):
        n = self._i.next().name
        return n, getattr(self, n)

    def keys(self):
        return self.__dict__.keys()

    def values(self):
        return self.__dict__.values()

    def items(self):
        return self.__dict__.items()

    def to_dict(self):
        return self.__dict__.copy()

    def to_dict_fields(self):
        """Returns a dictionary of just the db fields of this entity."""
        dict_fields = {'created': self.created_at,
                       'updated': self.updated_at,
                       'status': self.status}
        if self.deleted_at:
            dict_fields['deleted'] = self.deleted_at
        if self.deleted:
            dict_fields['is_deleted'] = True
        dict_fields.update(self._do_extra_dict_fields())
        return dict_fields

    def _do_extra_dict_fields(self):
        """Sub-class hook method: return dict of fields."""
        return {}


class TenantSecret(BASE, ModelBase):
    """
    Represents an association between a Tenant and a Secret.
    """

    __tablename__ = 'tenant_secret'

    tenant_id = Column(Integer, ForeignKey('tenants.id'), primary_key=True)
    secret_id = Column(Integer, ForeignKey('secrets.id'), primary_key=True)
    role = Column(String(255))
    secret = relationship("Secret", backref="tenant_assocs")


class Tenant(BASE, ModelBase):
    """
    Represents a Tenant in the datastore

    Tenants are users that wish to store secret information within
    Cloudkeep's Barbican.
    """

    __tablename__ = 'tenants'

    keystone_id = Column(String(255), unique=True)

    orders = relationship("Order", backref="tenant")
    secrets = relationship("TenantSecret", backref="tenants")

    def _do_extra_dict_fields(self):
        """Sub-class hook method: return dict of fields."""
        return {'keystone_id': self.keystone_id}


class Secret(BASE, ModelBase):
    """
    Represents a Secret in the datastore

    Secrets are any information Tenants wish to store within
    Cloudkeep's Barbican, though the actual encrypted data
    is stored in one or more EncryptedData entities on behalf
    of a Secret.

    Note that the mime_type here is the 'master' MIME type for
    the secret, which is used for PUTS and POSTS only. Barbican
    may then produce other MIME representations for the secret
    which are then stored as the EncryptedDatum for this secret,
    hence the need for EncryptedDatum records to have their own
    mime_type attributes.
    """

    __tablename__ = 'secrets'

    name = Column(String(255))
    expiration = Column(DateTime, default=None)
    mime_type = Column(String(255), nullable=False)
    algorithm = Column(String(255))
    bit_length = Column(Integer)
    cypher_type = Column(String(255))

    # TODO: Performance - Consider avoiding full load of all
    #   datum attributes here.
    encrypted_data = relationship("EncryptedDatum", lazy='joined')

    def __init__(self, parsed_request):
        """Creates secret from a dict."""
        super(Secret, self).__init__()

        self.name = parsed_request['name']
        self.mime_type = parsed_request['mime_type']

        self.expiration = parsed_request.get('expiration', None)
        self.algorithm = parsed_request.get('algorithm', None)
        self.bit_length = parsed_request.get('bit_length', None)
        self.cypher_type = parsed_request.get('cypher_type', None)

        self.status = States.ACTIVE

    def _do_delete_children(self, session):
        """
        Sub-class hook: delete children relationships.
        """
        for datum in self.encrypted_data:
            datum.delete(session)

    def _do_extra_dict_fields(self):
        """Sub-class hook method: return dict of fields."""
        return {'secret_id': self.id,
                'name': self.name,
                'expiration': self.expiration,
                'mime_type': self.mime_type,
                'algorithm': self.algorithm,
                'bit_length': self.bit_length,
                'cypher_type': self.cypher_type}


class EncryptedDatum(BASE, ModelBase):
    """
    Represents a the encrypted data for a Secret.

    Note that the mime_type below may or may not match that in the Secret
    record (see the Secret docstring for more details)
    """

    __tablename__ = 'encrypted_data'

    secret_id = Column(String(36), ForeignKey('secrets.id'),
                       nullable=False)

    mime_type = Column(String(255))
    cypher_text = Column(LargeBinary)
    kek_metadata = Column(Text)

    def __init__(self, secret=None):
        """Creates encrypted datum from a secret."""
        super(EncryptedDatum, self).__init__()

        if secret:
            self.secret_id = secret.id
            self.mime_type = secret.mime_type

        self.status = States.ACTIVE

    def _do_extra_dict_fields(self):
        """Sub-class hook method: return dict of fields."""
        return {'name': self.name,
                'mime_type': self.mime_type,
                'cypher_text': self.secret,
                'kek_metadata': self.kek_metadata}


class Order(BASE, ModelBase):
    """
    Represents an Order in the datastore

    Orders are requests for Barbican to create secret information,
    ranging from simple AES key generation requests to automated
    requests to Certificate Authorities to generate SSL certificates.
    """

    __tablename__ = 'orders'

    tenant_id = Column(String(36), ForeignKey('tenants.id'),
                       nullable=False)

    secret_name = Column(String(255))
    secret_algorithm = Column(String(255))
    secret_bit_length = Column(Integer)
    secret_cypher_type = Column(String(255))
    secret_mime_type = Column(String(255))
    secret_expiration = Column(DateTime, default=None)

    secret_id = Column(String(36), ForeignKey('secrets.id'),
                       nullable=True)

    def _do_extra_dict_fields(self):
        """Sub-class hook method: return dict of fields."""
        return {'secret': {'name': self.secret_name,
                           'mime_type': self.secret_mime_type,
                           'algorithm': self.secret_algorithm,
                           'bit_length': self.secret_bit_length,
                           'cypher_type': self.secret_cypher_type,
                           'expiration': self.secret_expiration},
                'secret_id': self.secret_id,
                'order_id': self.id}


# Keep this tuple synchronized with the models in the file
MODELS = [TenantSecret, Tenant, Secret, EncryptedDatum, Order]


def register_models(engine):
    """
    Creates database tables for all models with the given engine
    """
    LOG.debug("Models: {0}".format(repr(MODELS)))
    for model in MODELS:
        model.metadata.create_all(engine)


def unregister_models(engine):
    """
    Drops database tables for all models with the given engine
    """
    for model in MODELS:
        model.metadata.drop_all(engine)
